"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const path = require('path');
const fs = require('fs-extra-promise');
const child_process = require('child_process');
// let hook:PublishFileHook;
let hook = {
    filter: (env) => __awaiter(this, void 0, void 0, function* () {
        let trimFilePath = env.filePath.trim();
        return trimFilePath.indexOf(".png") !== -1 ||
            trimFilePath.indexOf(".jpg") !== -1 ||
            trimFilePath.indexOf(".jpeg") !== -1;
    }),
    execute: (env) => __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            let platformDir = path.join(__dirname, "../../../libpvr", `${process.platform}_${process.arch}`);
            if (fs.existsSync(platformDir)) {
                let execCommand = process.platform === "win32" ? "PVRTexToolCLI.exe" : "./PVRTexToolCLI";
                let rawFilePath = path.join(env.sourceRoot, "resource", env.filePath);
                let rawFileName = path.basename(rawFilePath);
                let pureName = path.basename(rawFilePath, path.extname(rawFileName));
                let newFilePath = path.join(path.dirname(rawFilePath), `${pureName}.pvr`);
                console.log("rawFilePath", rawFilePath);
                console.log("rawFileName", rawFileName);
                console.log("pureName", pureName);
                console.log("newFilePath", newFilePath);
                let params = [
                    "-i", rawFilePath,
                    "-f", "pvrtc1_4",
                    "-o", newFilePath];
                let spawn = child_process.spawn(execCommand, params, {
                    cwd: platformDir,
                    env: process.env
                });
                spawn.stdout.on("data", data => {
                    console.log(data);
                });
                spawn.stdout.on("error", data => {
                    console.error("onErr", data);
                    reject(data);
                });
                spawn.stderr.on("data", data => {
                    console.log(data.toString());
                });
                spawn.stderr.on("error", data => {
                    console.error("onErr", data);
                    reject(data);
                });
                spawn.on("close", (code) => {
                    console.log("pvr close:" + code);
                    if (code === 0) {
                        fs.readFileAsync(newFilePath).then(data => {
                            console.log("onClose:", data);
                            resolve(data);
                        }).catch(err => {
                            console.log("onErr", err);
                            reject(err);
                        });
                    }
                    else {
                        reject(code);
                    }
                });
            }
            else {
                reject(`libpvr DIR:${platformDir}\n NOT FOUND!`);
            }
        });
    })
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = hook;
//# sourceMappingURL=index.js.map